#!/usr/bin/python3
#
#       contRun_altnSource_cht.py
#
#       OpenFOAMが出力した熱流束から、FrontISTRを実行して、温度を出力する。
#       contRun_couplingSource_cht.pyを交互計算用に修正
#
#   23/10/21    saveMappingTemp:交互計算に対応させる為、
#               tempのfile名のstep数をendStepからtimeIndexに変更
#               addTempRenameRes,setDeltaTInCnt_cht,deleteResSetRestartFileCht:
#               大幅修正。
#               createCommTimeArea:新規追加。（前回のcouplingTime取得の為）
#               runFistr:Fistr起動前に、Fistrが停止している事を確認後、Fistrを起動
#      11/06    chtCommonAlternateモジュール使用に変更
#      11/16    allCommonConcAltnモジュール使用に変更
#      11/19    coupling_cht:温度の計算方法変更。
#               逐次計算→restartによる連続計算
#

import sys, os
import shutil
import pyCoupling
import chtCommonAlternate as chtComm
import allCommonConcAltn as allComm

waitCount = 5000
waitTime = 0.01


#------------------
#  coupling_cht
#------------------
def coupling_cht():
    """ OpenFOAMが出力した熱流束から、FrontISTRが温度を計算する。"""
    solidCaseDir = couplingDict["solidCaseDir"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    procNo = couplingDict["procNo"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    #---- heatFluxをmapping ---------
    #heatFluxの点群ファイル作成
    htFile = dataDir + "/" + patchName + "_faceHeatFlux_" + timeName + "_" + procNo + "tm"
    chtComm.createHeatFluxPtcFile(couplingDict, htFile)
    if int(procNo) == 0:
        #heatFluxをmappingしてcntファイルに反映
        chtComm.createMappingHeatFlux(couplingDict)
        #---- fistr実行 --------------
        #fsi計算前の設定
        setFsiChtCntFile("cht")
        bfrStep, bfrTime = mmCommTempArea.getBeforeCouplingTime()
        endStep, endTime = chtComm.setDeltaTInCnt_cht(couplingDict, bfrStep, bfrTime)
        #Fistr起動
        runFistr("cht", endStep, endTime)
        print("(py) --- creating temperature PTC ...", flush=True)
        #--- 温度を取得 ---------
        #温度のptcデータを作成
        np = pyCoupling.getNumProcsFistrFromDat(solidCaseDir)
        tempDict = chtComm.getTempDictFromResFile(couplingDict, endStep, np)
        tempPtc = chtComm.createTempPtc(couplingDict, tempDict)
        #温度をmappingしたfileを作成
        print("(py) --- temperature value is mapping to patch '" + patchName + "' ...", flush=True)
        chtComm.createMappingTempFile(couplingDict, tempPtc, endStep)
        #--- 結果file整理（file削除、restart設定）----
        chtComm.deleteResSetRestartFileCht(couplingDict, bfrStep, bfrTime, np)
    return ""

#
#  setFsiChtCntFile
def setFsiChtCntFile(sect):
    """ sectに応じたdatファイルに修正"""
    solidCaseDir = couplingDict["solidCaseDir"]
    pasteDatFile = solidCaseDir + os.sep + "hecmw_ctrl.dat"
    if sect == "fsi":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_fsi.dat"
    elif sect == "cht":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_cht.dat"
    elif sect == "chtss":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_chtss.dat"
    shutil.copy(datFile, pasteDatFile)

#
#  runFistr
#----------
def runFistr(sect, endStep, endTime):
    """ FrontISTRを起動"""
    allComm.runFistr(couplingDict, mmCommArea, sect, endStep, endTime)
    return


if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    currTime = sys.argv[1]      #currTime
    beforeTime = sys.argv[2]    #前回のtime
    procNo = sys.argv[3]        #procNo
    nProcs = sys.argv[4]        #nProcs
    timeIndex = sys.argv[5]     #timeIndex
    fluidCaseDir = os.getcwd()
    couplingData = pyCoupling.couplingData(fluidCaseDir)
    couplingDict = couplingData.read()
    couplingDict["fluidCaseDir"] = fluidCaseDir
    couplingDict["timeName"] = currTime
    couplingDict["beforeTime"] = beforeTime
    couplingDict["procNo"] = procNo
    couplingDict["nProcs"] = nProcs
    couplingDict["timeIndex"] = timeIndex
    waitCount = int(couplingDict["maxWaitTime"])
    #共有メモリの設定
    mmCommFile = allComm.createCommonArea(couplingDict)
    mmCommArea = allComm.mmCommonArea(couplingDict, mmCommFile)
    mmCommTempFile = allComm.createCommTimeTempArea(couplingDict)
    mmCommTempArea = allComm.mmCommonArea(couplingDict, mmCommTempFile)
    #couplingTimeを共有メモリに保存
    mmCommTempArea.saveCouplingTime()
    #現在時間が終了時間？
    if float(couplingDict["timeName"]) > float(couplingDict["endTime"]):
        exit()
    #beforeTimeが計算開始時間よりも前 or 現時点がbeforeTime？
    if (float(beforeTime) <  float(couplingDict["startTime"]) or    #restart初回
        float(beforeTime) == float(currTime)):                      #計算初回
        exit() 
    #mainへ
    stat = coupling_cht()
    if stat != "":
        raise NameError(stat)
